@@ -109,6 +109,15 @@ on_aboutdialog_klavaro_response (GtkDialog *dialog, gint response_id, gpointer u
 	gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
+G_MODULE_EXPORT void
+on_checkbutton_speech_toggled (GtkToggleButton * togglebutton, gpointer user_data)
+{
+	if (callbacks_shield)
+		return;
+	main_preferences_set_boolean ("interface", "speech",
+		       	gtk_toggle_button_get_active (togglebutton));
+}
+
 G_MODULE_EXPORT void
 on_combobox_language_changed (GtkComboBox *cmb, gpointer user_data)
 {
@@ -185,65 +194,129 @@ G_MODULE_EXPORT void
 on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 {
 	gboolean beep;
+	gchar *tmp;
+	gchar *search;
 	gchar *tmp_font;
-	gchar *color_cursor_blink_fg;
-	gchar *color_cursor_blink_bg;
+	gchar *color_fg;
+	gchar *color_bg;
+	gchar *color_main_fg;
+	gchar *color_main_bg;
 	GtkWidget *wg;
 	GtkTextBuffer *buf;
 	PangoFontDescription *font_desc;
 	GdkColor color;
 
+	/* Set main color for tutor text (char_untouched)
+	 */
+	if (main_preferences_exist ("colors", "char_untouched_bg"))
+		color_main_bg = main_preferences_get_string ("colors", "char_untouched_bg");
+	else
+		color_main_bg = g_strdup (TUTOR_CREAM);
+	if (main_preferences_exist ("colors", "char_untouched_fg"))
+		color_main_fg = main_preferences_get_string ("colors", "char_untouched_fg");
+	else
+		color_main_fg = g_strdup (TUTOR_BLACK);
 	/*
-	 * Colors of text on the tutor window
+	 * Colors of text on the tutor window (note: ordering here matters, the first tag created is in the bottom!)
 	 */
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
 
+	gtk_text_buffer_create_tag (buf, "char_keep_wrap",
+		       	"background", color_main_bg,
+		       	"foreground", color_main_fg,
+		       	"underline", PANGO_UNDERLINE_NONE, NULL);
+
+	gtk_text_buffer_create_tag (buf, "char_keep_wrap2",
+		       	"background", color_main_bg,
+		       	"foreground", color_main_fg,
+		       	"underline", PANGO_UNDERLINE_NONE, NULL);
+
 	gtk_text_buffer_create_tag (buf, "char_untouched",
-		       	"background", TUTOR_CREAM,
-		       	"foreground", TUTOR_BLACK,
+		       	"background", color_main_bg,
+		       	"foreground", color_main_fg,
 		       	"underline", PANGO_UNDERLINE_NONE, NULL);
 
-	gtk_text_buffer_create_tag (buf, "char_wrong",
-		       	"background", TUTOR_RED_LITE,
-		       	"foreground", TUTOR_RED,
-		       	"underline", PANGO_UNDERLINE_NONE, NULL);
-
-	gtk_text_buffer_create_tag (buf, "char_correct",
-		       	"background", TUTOR_CREAM,
-		       	"foreground", TUTOR_GREEN,
-		       	"underline", PANGO_UNDERLINE_NONE, NULL);
-
-	gtk_text_buffer_create_tag (buf, "char_retouched",
-			    "background", TUTOR_GRAY,
-			    "foreground", TUTOR_BROWN,
-			    "underline", PANGO_UNDERLINE_NONE, NULL);
-
-	color_cursor_blink_bg = main_preferences_get_string ("colors", "cursor_blink_bg");
-	color_cursor_blink_fg = main_preferences_get_string ("colors", "cursor_blink_fg");
-	if (color_cursor_blink_fg && color_cursor_blink_bg)
-	{
-		gtk_text_buffer_create_tag (buf, "cursor_blink",
-			       	"background", color_cursor_blink_bg,
-			       	"foreground", color_cursor_blink_fg,
-			       	"underline", PANGO_UNDERLINE_NONE, NULL);
-	}
+	/* char_wrong */
+	if (main_preferences_exist ("colors", "char_wrong_bg"))
+		color_bg = main_preferences_get_string ("colors", "char_wrong_bg");
 	else
-	{
-		gtk_text_buffer_create_tag (buf, "cursor_blink",
-			       	"background", TUTOR_YELLOW,
-			       	"foreground", TUTOR_BLACK,
-			       	"underline", PANGO_UNDERLINE_NONE, NULL);
-	}
-	if (color_cursor_blink_fg)
-		g_free (color_cursor_blink_fg);
-	if (color_cursor_blink_bg)
-		g_free (color_cursor_blink_bg);
-
-	gtk_text_buffer_create_tag (buf, "text_intro",
-		       	"background", TUTOR_WHITE,
-		       	"foreground", TUTOR_BLACK,
+		color_bg = g_strdup (TUTOR_RED_LITE);
+	if (main_preferences_exist ("colors", "char_wrong_fg"))
+		color_fg = main_preferences_get_string ("colors", "char_wrong_fg");
+	else
+		color_fg = g_strdup (TUTOR_RED);
+	gtk_text_buffer_create_tag (buf, "char_wrong",
+		       	"background", color_bg,
+		       	"foreground", color_fg,
 		       	"underline", PANGO_UNDERLINE_NONE, NULL);
+	g_free (color_bg);
+	g_free (color_fg);
 
+	/* char_correct */
+	if (main_preferences_exist ("colors", "char_correct_bg"))
+		color_bg = main_preferences_get_string ("colors", "char_correct_bg");
+	else
+		color_bg = g_strdup (TUTOR_CREAM);
+	if (main_preferences_exist ("colors", "char_correct_fg"))
+		color_fg = main_preferences_get_string ("colors", "char_correct_fg");
+	else
+		color_fg = g_strdup (TUTOR_GREEN);
+	gtk_text_buffer_create_tag (buf, "char_correct",
+		       	"background", color_bg,
+		       	"foreground", color_fg,
+		       	"underline", PANGO_UNDERLINE_NONE, NULL);
+	g_free (color_bg);
+	g_free (color_fg);
+
+	/* char_retouched */
+	if (main_preferences_exist ("colors", "char_retouched_bg"))
+		color_bg = main_preferences_get_string ("colors", "char_retouched_bg");
+	else
+		color_bg = g_strdup (TUTOR_GRAY);
+	if (main_preferences_exist ("colors", "char_retouched_fg"))
+		color_fg = main_preferences_get_string ("colors", "char_retouched_fg");
+	else
+		color_fg = g_strdup (TUTOR_BROWN);
+	gtk_text_buffer_create_tag (buf, "char_retouched",
+			    "background", color_bg,
+			    "foreground", color_fg,
+			    "underline", PANGO_UNDERLINE_NONE, NULL);
+	g_free (color_bg);
+	g_free (color_fg);
+
+	/* cursor_blink */
+	if (main_preferences_exist ("colors", "cursor_blink_bg"))
+		color_bg = main_preferences_get_string ("colors", "cursor_blink_bg");
+	else
+		color_bg = g_strdup (TUTOR_YELLOW);
+	if (main_preferences_exist ("colors", "cursor_blink_fg"))
+		color_fg = main_preferences_get_string ("colors", "cursor_blink_fg");
+	else
+		color_fg = g_strdup (TUTOR_BLACK);
+	gtk_text_buffer_create_tag (buf, "cursor_blink",
+			"background", color_bg,
+			"foreground", color_fg,
+			"underline", PANGO_UNDERLINE_NONE, NULL);
+	g_free (color_bg);
+	g_free (color_fg);
+
+	/* text_intro */
+	if (main_preferences_exist ("colors", "text_intro_bg"))
+		color_bg = main_preferences_get_string ("colors", "text_intro_bg");
+	else
+		color_bg = g_strdup (TUTOR_WHITE);
+	if (main_preferences_exist ("colors", "text_intro_fg"))
+		color_fg = main_preferences_get_string ("colors", "text_intro_fg");
+	else
+		color_fg = g_strdup (TUTOR_BLACK);
+	gtk_text_buffer_create_tag (buf, "text_intro",
+		       	"background", color_bg,
+		       	"foreground", color_fg,
+		       	"underline", PANGO_UNDERLINE_NONE, NULL);
+	g_free (color_bg);
+	g_free (color_fg);
+
+	/* Tutor font */
 	tmp_font = main_preferences_get_string ("tutor", "lesson_font");
 	if (tmp_font == NULL)
 	{
@@ -258,25 +331,36 @@ on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 	gtk_widget_modify_font (widget, font_desc);
 	pango_font_description_free (font_desc);
 
-	/* Change default text color throughout the widget */
-	gdk_color_parse (TUTOR_BLACK, &color);
-	gtk_widget_modify_text (widget, GTK_STATE_INSENSITIVE, &color);
-
 	/* Change default background color throughout the widget */
-	gdk_color_parse (TUTOR_CREAM, &color);
+	gdk_color_parse (color_main_bg, &color);
 	gtk_widget_modify_base (widget, GTK_STATE_INSENSITIVE, &color);
 
-	/*
-	 * Turns on/off the beeps according to last time
+	/* Change default text color throughout the widget */
+	gdk_color_parse (color_main_fg, &color);
+	gtk_widget_modify_text (widget, GTK_STATE_INSENSITIVE, &color);
+
+	/* Turns on/off the beeps according to last time
 	 */
 	if (main_preferences_exist ("tutor", "tutor_beep"))
 		beep = main_preferences_get_boolean ("tutor", "tutor_beep");
 	else
 		beep = TRUE;
-
 	wg = get_wg ("togglebutton_tutor_beep");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wg), beep);
 	main_preferences_set_boolean ("tutor", "tutor_beep", beep);
+
+	/* Workaround to extend the editable lessons, from 50 to 60,
+	 * avoiding to change the translatable message in the tooltip.
+	 */
+	tmp = gtk_widget_get_tooltip_text (get_wg ("togglebutton_edit_basic_lesson"));
+	search = g_strstr_len (tmp, -1, "50");
+	if (search)
+		search[0] = '6';
+	gtk_widget_set_tooltip_text (get_wg ("togglebutton_edit_basic_lesson"), tmp);
+	g_free (tmp);
+
+	g_free (color_main_bg);
+	g_free (color_main_fg);
 }
 
 static void
@@ -1019,24 +1103,15 @@ G_MODULE_EXPORT void
 on_virtual_key_changed (GtkEditable *edit, gpointer user_data)
 {
 	gunichar ch;
-	gboolean go_next = TRUE;
 
 	if (callbacks_shield)
 		return;
 
 	ch = g_utf8_get_char_validated (gtk_entry_get_text (GTK_ENTRY (edit)), -1);
 	if (ch == (gunichar)-2 || ch == (gunichar)-1 || ch == 0)
-	{
 		ch = L' ';
-		go_next = FALSE;
-	}
 
 	keyb_change_key (ch);
-
-	/*
-	if (go_next)
-		keyb_edit_next ();
-	*/
 }
 
 /* Confirm-dialog shared tasks
@@ -1221,12 +1296,13 @@ on_combobox_stat_module_changed (GtkComboBox *cmb, gpointer user_data)
 	g_free (tmp);
 	switch (gtk_combo_box_get_active (cmb))
 	{
-		/* Uncomment this in order to enable the error plottings
-		*/
 	case 1:
 		tmp = g_strdup_printf ("%s", _("Errors"));
 		gtk_combo_box_insert_text (GTK_COMBO_BOX (get_wg ("combobox_stat_type")), 2, tmp);
 		g_free (tmp);
+		tmp = g_strdup_printf ("%s", _("Touch times (s)"));
+		gtk_combo_box_insert_text (GTK_COMBO_BOX (get_wg ("combobox_stat_type")), 3, tmp);
+		g_free (tmp);
 		break;
 	case 3:
 		tmp = g_strdup_printf ("%s (%%)", _("Fluidity"));
@@ -1262,9 +1338,10 @@ on_combobox_stat_type_changed (GtkComboBox *cmb, gpointer user_data)
 	if (active < 0)
 		return;
 
-	if (gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_module"))) == 1 &&
-			active == 2)
-		plot_draw_chart (6);
+	/* Adapt module (1) has special charts
+	 */
+	if (gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_module"))) == 1)
+		plot_draw_chart (active + (active < 2 ? 1 : 4));
 	else
 		plot_draw_chart (active + 1);
 }
