@@ -58,14 +58,17 @@ cursor_advance (gint n)
 	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 
+	/* Get current position for the iter old_start */
 	gtk_text_buffer_get_iter_at_mark (buf, &new_start, gtk_text_buffer_get_insert (buf));
 	old_start = new_start;
 
+	/* Get new position for the iter new_start */
 	if (n > 0)
 		for (i = 0; i < n && gtk_text_iter_forward_cursor_position (&new_start); i++);
 	else
 		for (i = 0; i > n && gtk_text_iter_backward_cursor_position (&new_start); i--);
 
+	/* Move cursor blinking */
 	end = new_start;
 	gtk_text_iter_forward_char (&end);
 	gtk_text_buffer_apply_tag_by_name (buf, "cursor_blink", &new_start, &end);
@@ -74,13 +77,14 @@ cursor_advance (gint n)
 	gtk_text_iter_forward_char (&end);
 	gtk_text_buffer_remove_tag_by_name (buf, "cursor_blink", &old_start, &end);
 
+	/* Move cursor */
 	gtk_text_buffer_place_cursor (buf, &new_start);
 
+	/* Check need for auto-scrolling */
 	if (i == n)
 	{
 		end = new_start;
 		gtk_text_iter_forward_line (&end);
-		//gtk_text_iter_forward_sentence_ends (&end, 3);
 		mark = gtk_text_buffer_create_mark (buf, "aux", &end, FALSE);
 		cursor_out_screen = gtk_text_view_move_mark_onscreen (GTK_TEXT_VIEW (wg), mark);
 		if (cursor_out_screen)
@@ -217,7 +221,7 @@ cursor_switch_on ()
 	gtk_text_buffer_get_iter_at_mark (buf, &start, gtk_text_buffer_get_insert (buf));
 	gtk_text_buffer_get_iter_at_mark (buf, &end, gtk_text_buffer_get_insert (buf));
 	gtk_text_iter_forward_char (&end);
-	if (tutor_get_correcting () || tutor_get_type () != TT_FLUID)
+	if (tutor_get_correcting ())
 		gtk_text_buffer_apply_tag_by_name (buf, "cursor_blink", &start, &end);
 	else
 		gtk_text_buffer_remove_tag_by_name (buf, "cursor_blink", &start, &end);
@@ -240,7 +244,7 @@ cursor_switch_off ()
 	gtk_text_buffer_get_iter_at_mark (buf, &start, gtk_text_buffer_get_insert (buf));
 	gtk_text_buffer_get_iter_at_mark (buf, &end, gtk_text_buffer_get_insert (buf));
 	gtk_text_iter_forward_char (&end);
-	if (tutor_get_correcting () || tutor_get_type () != TT_FLUID)
+	if (tutor_get_correcting ())
 		gtk_text_buffer_remove_tag_by_name (buf, "cursor_blink", &start, &end);
 	else
 		gtk_text_buffer_apply_tag_by_name (buf, "cursor_blink", &start, &end);
