@@ -39,6 +39,7 @@ static struct
 	gunichar lochars[4][KEY_LINE_LEN + 1];
 	gunichar upchars[4][KEY_LINE_LEN + 1];
 	GtkWidget *but[4][KEY_LINE_LEN - 1];
+	GtkWidget *lab[4][KEY_LINE_LEN - 1];
 	GtkWidget *entry;
 	struct
 	{
@@ -203,13 +204,32 @@ void
 keyb_create_virtual_keys ()
 {
 	gint i, j;
+	gchar *hlp;
 	GtkFixed *fix;
 	GdkColor color;
 	
-	gdk_color_parse (KEYB_PURPLE, &color);
+	/* Set color of space key
+	 */
+	if (main_preferences_exist ("colors", "key_5"))
+		hlp = main_preferences_get_string ("colors", "key_5");
+	else
+		hlp = g_strdup (KEYB_PURPLE);
+	gdk_color_parse (hlp, &color);
+	gtk_widget_modify_bg (get_wg ("but_space"), GTK_STATE_NORMAL, &color);
+	g_free (hlp);
 
+	/* Set text color of keys
+	 */
+	if (main_preferences_exist ("colors", "key_fg"))
+		hlp = main_preferences_get_string ("colors", "key_fg");
+	else
+		hlp = g_strdup (KEYB_BLACK);
+	gdk_color_parse (hlp, &color);
+	g_free (hlp);
+
+	/* Create and position buttons and labels
+	 */
 	fix = GTK_FIXED (get_wg ("fixed_keyboard"));
-
 	for (i = 0; i < 4; i++)
 	{
 		for (j = 0; j < KEY_LINE_LEN - 1; j++)
@@ -221,8 +241,11 @@ keyb_create_virtual_keys ()
 				       	G_CALLBACK (on_virtual_key_clicked), NULL);
   			g_signal_connect_after ((gpointer) keyb.but[i][j], "grab-focus",
 				       	G_CALLBACK (on_virtual_key_grab_focus), NULL);
-			//gtk_widget_modify_bg (keyb.but[i][j], GTK_STATE_NORMAL, &color);
-			//gtk_widget_modify_bg (keyb.but[i][j], GTK_STATE_INSENSITIVE, &color);
+			keyb.lab[i][j] = gtk_label_new ("0");
+			gtk_widget_modify_fg (keyb.lab[i][j], GTK_STATE_NORMAL, &color);
+			gtk_widget_modify_fg (keyb.lab[i][j], GTK_STATE_PRELIGHT, &color);
+			gtk_container_add (GTK_CONTAINER (keyb.but[i][j]), keyb.lab[i][j]);
+
 			if (i > 0)
 			{
 				if (i == 1)
@@ -237,6 +260,7 @@ keyb_create_virtual_keys ()
 				}
 			}
 			gtk_widget_show (keyb.but[i][j]);
+			gtk_widget_show (keyb.lab[i][j]);
 		}
 	}
 	gtk_widget_set_size_request (keyb.but[1][12], 53, 32);
@@ -250,8 +274,6 @@ keyb_create_virtual_keys ()
 	gtk_entry_set_alignment (GTK_ENTRY (keyb.entry), 0.5);
 	g_object_set (GTK_OBJECT (keyb.entry), "shadow-type", GTK_SHADOW_NONE, NULL);
   	g_signal_connect_after ((gpointer) keyb.entry, "changed", G_CALLBACK (on_virtual_key_changed), NULL);
-
-	gtk_widget_modify_bg (get_wg ("but_space"), GTK_STATE_NORMAL, &color);
 }
 
 /**********************************************************************
@@ -568,7 +590,7 @@ keyb_update_virtual_layout ()
 			    && g_unichar_tolower (keyb.upchars[i][j]) == keyb.lochars[i][j])
 				uch = g_unichar_toupper (uch);
 			ut8[g_unichar_to_utf8 (uch, ut8)] = '\0';
-			gtk_button_set_label (GTK_BUTTON (keyb.but[i][j]), ut8);
+			gtk_label_set_text (GTK_LABEL (keyb.lab[i][j]), ut8);
 		}
 	}
 }
@@ -1034,6 +1056,7 @@ keyb_intro_step (gint step)
 	GtkLabel *tx1;
 	GtkLabel *tx2;
 	GtkTextBuffer *buffer;
+	GtkWidget *wg;
 
 	static gchar *intro01 = NULL;
 	static gchar *intro02 = NULL;
@@ -1197,6 +1220,11 @@ keyb_intro_step (gint step)
 	}
 	g_free (intro00);
 
+	/* Blind people want no fancy autonomous buttons jumping around */
+	wg = get_wg ("checkbutton_speech");
+	if (GTK_WIDGET_VISIBLE (wg) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wg)))
+		hints_demo_fingers (0);
+
 	if (step == 0)
 		gtk_widget_set_sensitive (get_wg ("button_keyboard_previous"), FALSE);
 	else
@@ -1389,7 +1417,8 @@ keyb_edit_button (GtkButton * button)
 	callbacks_shield_set (TRUE);
 
 	gtk_widget_grab_focus (GTK_WIDGET (button));
-	gtk_entry_set_text (GTK_ENTRY (keyb.entry),  gtk_button_get_label (button));
+	gtk_entry_set_text (GTK_ENTRY (keyb.entry),
+		       	gtk_label_get_text (GTK_LABEL (keyb.lab[keyb.pos.i][keyb.pos.j])));
 	gtk_fixed_move (GTK_FIXED (get_wg ("fixed_keyboard")), keyb.entry,
 		       	2 + x0[keyb.pos.i] + KEY_DX * keyb.pos.j,
 		       	2 + KEY_DY * keyb.pos.i);
@@ -1470,7 +1499,7 @@ keyb_change_key (gunichar real_key)
 
 	str_char = g_unichar_toupper (real_key);
 	tmp_utf8[g_unichar_to_utf8 (str_char, tmp_utf8)] = '\0';
-	gtk_button_set_label (GTK_BUTTON (keyb.but[key_lin][key_col]), tmp_utf8);
+	gtk_label_set_text (GTK_LABEL (keyb.lab[key_lin][key_col]), tmp_utf8);
 
 	wg = get_wg ("toggle_shift1");
 	tog_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wg));
@@ -1595,32 +1624,70 @@ hints_string_from_charcode (gchar charcode)
 gchar *
 hints_color_from_charcode (gchar charcode)
 {
-	gchar *color = NULL;
+	static gchar *hlp = NULL;
+
+	g_free (hlp);
 
 	switch (charcode)
 	{
 	case '1':
-		return KEYB_BLUE;
+		if (main_preferences_exist ("colors", "key_1"))
+			hlp = main_preferences_get_string ("colors", "key_1");
+		else
+			hlp = g_strdup (KEYB_BLUE);
+		break;
 	case '2':
-		return KEYB_RED;
+		if (main_preferences_exist ("colors", "key_2"))
+			hlp = main_preferences_get_string ("colors", "key_2");
+		else
+			hlp = g_strdup (KEYB_RED);
+		break;
 	case '3':
-		return KEYB_GREEN;
+		if (main_preferences_exist ("colors", "key_3"))
+			hlp = main_preferences_get_string ("colors", "key_3");
+		else
+			hlp = g_strdup (KEYB_GREEN);
+		break;
 	case '4':
-		return KEYB_YELLOW;
+		if (main_preferences_exist ("colors", "key_4"))
+			hlp = main_preferences_get_string ("colors", "key_4");
+		else
+			hlp = g_strdup (KEYB_YELLOW);
+		break;
 	case '5':
-		return KEYB_PURPLE;
+		if (main_preferences_exist ("colors", "key_5"))
+			hlp = main_preferences_get_string ("colors", "key_5");
+		else
+			hlp = g_strdup (KEYB_PURPLE);
+		break;
 	case '6':
-		return KEYB_ORANGE;
+		if (main_preferences_exist ("colors", "key_6"))
+			hlp = main_preferences_get_string ("colors", "key_6");
+		else
+			hlp = g_strdup (KEYB_ORANGE);
+		break;
 	case '7':
-		return KEYB_GREEN;
+		if (main_preferences_exist ("colors", "key_7"))
+			hlp = main_preferences_get_string ("colors", "key_7");
+		else
+			hlp = g_strdup (KEYB_GREEN);
+		break;
 	case '8':
-		return KEYB_RED;
+		if (main_preferences_exist ("colors", "key_8"))
+			hlp = main_preferences_get_string ("colors", "key_8");
+		else
+			hlp = g_strdup (KEYB_RED);
+		break;
 	case '9':
-		return KEYB_BLUE;
+		if (main_preferences_exist ("colors", "key_9"))
+			hlp = main_preferences_get_string ("colors", "key_9");
+		else
+			hlp = g_strdup (KEYB_BLUE);
+		break;
 	default:
-		return "#AFAFAF";
+		hlp = g_strdup ("#AFAFAF");
 	}
-	return (color);
+	return hlp;
 }
 
 void
@@ -1711,7 +1778,7 @@ hints_update_from_button (GtkButton *button)
 }
 
 /* Update the image of the window_hints
- * Map the character to the file which shows the finger associated with that key
+ * Maps the character to the file which shows the finger associated with that key
  */
 void
 hints_update_from_char (gunichar character)
@@ -1788,3 +1855,28 @@ hints_demo_fingers (guint msec)
 		source = g_main_context_find_source_by_id (NULL, id);
 	}
 }
+
+gchar *
+hints_finger_name_from_char (gunichar uch)
+{
+	gint i, j;
+
+	if (uch == UPSYM || uch == L'\n' || uch == L'\r')
+		return (hints_string_from_charcode ('9'));
+	if (uch == L' ')
+		return (hints_string_from_charcode ('5'));
+	
+	hints_init (); // if already initialized, do nothing
+
+	for (i = 3; i >= 0; i--)
+		for (j = 0; j < 15; j++)
+			if (uch == keyb.lochars[i][j])
+				return (hints_string_from_charcode (hints[i][j]));
+
+	for (i = 3; i >= 0; i--)
+		for (j = 0; j < 15; j++)
+			if (uch == keyb.upchars[i][j])
+				return (hints_string_from_charcode (hints[i][j]));
+
+	return (g_strdup (" "));
+}
