@@ -17,6 +17,7 @@
 #include <string.h>
 #include <time.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <locale.h>
 #include <errno.h>
 #include <glib.h>
@@ -37,7 +38,7 @@
 #include "top10.h"
 #include "tutor.h"
 
-#define MAX_TOUCH_TICS 8000
+#define MAX_TOUCH_TICS 10000
 struct
 {
 	TutorType type;
@@ -86,6 +87,8 @@ tutor_set_query (TutorQuery query)
 gint
 tutor_get_correcting ()
 {
+	if (tutor.type != TT_FLUID)
+		return (FALSE);
 	return (tutor.correcting);
 }
 
@@ -158,8 +161,7 @@ tutor_init (TutorType tt_type)
 	/******************************
 	 * Set the layout for each exercise type
 	 */
-	//if (!UNIX_OK)
-		gtk_widget_hide (get_wg ("button_tutor_top10"));
+	gtk_widget_hide (get_wg ("button_tutor_top10"));
 	gtk_widget_hide (get_wg ("entry_custom_basic_lesson"));
 	if (tutor.type == TT_BASIC || tutor.type == TT_FLUID)
 	{
@@ -179,12 +181,11 @@ tutor_init (TutorType tt_type)
 		else
 		{
 			gtk_widget_hide (get_wg ("togglebutton_edit_basic_lesson"));
-			//if (UNIX_OK)
-				gtk_widget_show (get_wg ("button_tutor_top10"));
+			gtk_widget_show (get_wg ("button_tutor_top10"));
 			gtk_widget_hide (get_wg ("button_tutor_show_keyb"));
 			gtk_label_set_text (GTK_LABEL (get_wg ("label_lesson")), _("Paragraphs:"));
 			callbacks_shield_set (TRUE);
-			gtk_spin_button_set_range (GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson")), 1, 10);
+			gtk_spin_button_set_range (GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson")), 0, 10);
 			callbacks_shield_set (FALSE);
 		}
 	}
@@ -309,6 +310,7 @@ tutor_update_intro ()
 {
 	gchar *tmp_name;
 	gchar *text;
+	gchar *color_bg;
 	GdkColor color;
 	GtkWidget *wg;
 	GtkLabel *wg_label;
@@ -344,9 +346,14 @@ tutor_update_intro ()
 	gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (wg_text), &start, &end);
 	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "lesson_font", &start, &end);
 	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "text_intro", &start, &end);
-	gdk_color_parse (TUTOR_WHITE, &color);
+
+	if (main_preferences_exist ("colors", "text_intro_bg"))
+		color_bg = main_preferences_get_string ("colors", "text_intro_bg");
+	else
+		color_bg = g_strdup (TUTOR_WHITE);
+	gdk_color_parse (color_bg, &color);
 	gtk_widget_modify_base (get_wg ("text_tutor"), GTK_STATE_INSENSITIVE, &color);
-	
+	g_free (color_bg);
 
 	wg = get_wg ("scrolledwindow_tutor_main");
 	scroll = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (wg));
@@ -362,6 +369,7 @@ tutor_update_start ()
 {
 	gchar *tmp_name;
 	gchar *text;
+	gchar *color_bg;
 	GdkColor color;
 	GtkWidget *wg;
 	GtkTextBuffer *buf;
@@ -407,17 +415,30 @@ tutor_update_start ()
 		fluid_draw_random_paragraphs ();
 	}
 
-	//gtk_widget_set_sensitive (get_wg ("button_tutor_restart"), FALSE);
-
 	/*
 	 * Apply tutor background color and font to the text
 	 */
+	if (main_preferences_exist ("colors", "char_untouched_bg"))
+		color_bg = main_preferences_get_string ("colors", "char_untouched_bg");
+	else
+		color_bg = g_strdup (TUTOR_CREAM);
+	gdk_color_parse (color_bg, &color);
+	gtk_widget_modify_base (get_wg ("text_tutor"), GTK_STATE_INSENSITIVE, &color);
+	g_free (color_bg);
+
 	gtk_text_buffer_get_bounds (buf, &start, &end);
 	gtk_text_iter_backward_char (&end);
-	gtk_text_buffer_apply_tag_by_name (buf, "char_untouched", &start, &end);
 	gtk_text_buffer_apply_tag_by_name (buf, "lesson_font", &start, &end);
-	gdk_color_parse (TUTOR_CREAM, &color);
-	gtk_widget_modify_base (get_wg ("text_tutor"), GTK_STATE_INSENSITIVE, &color);
+	end = start;
+	while (gtk_text_iter_forward_word_end (&end))
+	{
+		gtk_text_buffer_apply_tag_by_name (buf, "char_keep_wrap", &start, &end);
+		start = end;
+		if (! gtk_text_iter_forward_char (&end))
+			break;
+		gtk_text_buffer_apply_tag_by_name (buf, "char_keep_wrap2", &start, &end);
+		start = end;
+	}
 	
 	tutor_message (_("Start typing when you are ready. "));
 
@@ -463,8 +484,23 @@ tutor_process_touch (gunichar user_chr)
 		gtk_text_buffer_place_cursor (wg_buffer, &start);
 		cursor_set_blink (TRUE);
 		cursor_on (NULL);
-		if (tutor.type == TT_BASIC)
+		accur_sort ();
+
+		switch (tutor.type)
+		{
+		case TT_BASIC:
 			hints_update_from_char (cursor_get_char ());
+			tutor_speak_char ();
+			return;
+		case TT_ADAPT:
+			tutor_speak_char ();
+			return;
+		case TT_VELO:
+			tutor_speak_word ();
+			return;
+		case TT_FLUID:
+			return;
+		}
 		return;
 
 	case QUERY_START:
@@ -479,14 +515,27 @@ tutor_process_touch (gunichar user_chr)
 		g_free (u8ch);
 		callbacks_shield_set (FALSE);
 
-		if (tutor.type == TT_BASIC)
-			hints_update_from_char (cursor_get_char ());
-
 		g_timer_start (tutor.tmr);
 		if (tutor.type == TT_FLUID)
 			tutor_eval_forward_backward (user_chr);
 		else
 			tutor_eval_forward (user_chr);
+
+		switch (tutor.type)
+		{
+		case TT_BASIC:
+			hints_update_from_char (cursor_get_char ());
+			tutor_speak_char ();
+			return;
+		case TT_ADAPT:
+			tutor_speak_char ();
+			return;
+		case TT_VELO:
+			tutor_speak_word ();
+			return;
+		case TT_FLUID:
+			return;
+		}
 		return;
 
 	case QUERY_END:
@@ -526,8 +575,22 @@ tutor_process_touch (gunichar user_chr)
 	else
 	{
 		cursor_on (NULL);
-		if (tutor.type == TT_BASIC)
+
+		switch (tutor.type)
+		{
+		case TT_BASIC:
 			hints_update_from_char (cursor_get_char ());
+			tutor_speak_char ();
+			return;
+		case TT_ADAPT:
+			tutor_speak_char ();
+			return;
+		case TT_VELO:
+			tutor_speak_word ();
+			return;
+		case TT_FLUID:
+			return;
+		}
 	}
 }
 
@@ -561,17 +624,27 @@ tutor_eval_forward (gunichar user_chr)
 	 */
 	if (user_chr == real_chr)
 	{
+		if (tutor.ttidx < MAX_TOUCH_TICS)
+		{
+			tutor.touch_time[tutor.ttidx] =
+				g_timer_elapsed (tutor.tmr, NULL) - tutor.touch_time[tutor.ttidx];
+			tutor.ttidx++;
+			tutor.touch_time[tutor.ttidx] = g_timer_elapsed (tutor.tmr, NULL);
+			if (tutor.type != TT_BASIC)
+				accur_correct (real_chr, tutor.touch_time[tutor.ttidx-1]);
+		}
+
 		cursor_paint_char ("char_correct");
-		if (tutor.type != TT_BASIC)
-			accur_correct (real_chr);
 	}
 	else
 	{
-		tutor_beep ();
-		cursor_paint_char ("char_wrong");
-		tutor.n_errors++;
+		tutor.touch_time[tutor.ttidx] = g_timer_elapsed (tutor.tmr, NULL);
 		if (tutor.type != TT_BASIC)
 			accur_wrong (real_chr);
+
+		cursor_paint_char ("char_wrong");
+		tutor.n_errors++;
+		tutor_beep ();
 	}
 
 
@@ -601,7 +674,7 @@ tutor_eval_forward_backward (gunichar user_chr)
 
 	/*
 	 * Work on backspaces
-	 * L'\t' means a super <Ctrl> + <Backspace>
+	 * L'\t' means an hyper <Ctrl> + <Backspace>
 	 */
 	if (user_chr == L'\b' || user_chr == L'\t')
 	{
@@ -626,7 +699,7 @@ tutor_eval_forward_backward (gunichar user_chr)
 		}
 
 		/*
-		 * Test begin of line
+		 * Test for start of line
 		 */
 		if (cursor_get_char () == L'\n')
 			if (cursor_advance (-1) != -1)
@@ -636,7 +709,7 @@ tutor_eval_forward_backward (gunichar user_chr)
 			}
 
 		/*
-		 * Reinitialize the color
+		 * Reinitialize the color (no visible effect at all...)
 		 */
 		cursor_paint_char ("char_untouched");
 
@@ -653,7 +726,7 @@ tutor_eval_forward_backward (gunichar user_chr)
 
 	real_chr = cursor_get_char ();
 
-	// Minimizing the line breaking bug:
+	// Minimizing the line-breaking bug:
 	if (user_chr == UPSYM && real_chr == L' ')
 		user_chr = L' ';
 
@@ -678,17 +751,17 @@ tutor_eval_forward_backward (gunichar user_chr)
 		else
 		{
 			cursor_paint_char ("char_correct");
-			accur_correct (real_chr);
+			accur_correct (real_chr, tutor.touch_time[tutor.ttidx-1]);
 		}
 	}
 	else
 	{
 		tutor.touch_time[tutor.ttidx] = g_timer_elapsed (tutor.tmr, NULL);
 		cursor_paint_char ("char_wrong");
-		tutor_beep ();
 		tutor.retro_pos++;
 		if (tutor.retro_pos == 1)
 			accur_wrong (real_chr);
+		tutor_beep ();
 	}
 
 	if (tutor.correcting > 0)
@@ -1334,42 +1407,6 @@ tutor_draw_paragraph (gchar * utf8_text)
 	gtk_text_buffer_insert_at_cursor (buf, tmp2, -1);
 }
 
-/**********************************************************************
- * Put 'mesg' in the message entry line of the shared tutor window
- */
-void
-tutor_message (gchar * mesg)
-{
-	gint pos = 0;
-	GtkWidget *wg;
-
-	if (mesg == NULL)
-	{
-		g_message ("tutor_message() --> not showing NULL message!");
-		return;
-	}
-
-	wg = get_wg ("entry_mesg");
-	callbacks_shield_set (TRUE);
-	gtk_editable_delete_text (GTK_EDITABLE (wg), 0, -1);
-	gtk_editable_insert_text (GTK_EDITABLE (wg), g_strdup (mesg), strlen (mesg), &pos);
-	gtk_editable_set_position (GTK_EDITABLE (wg), -1);
-	callbacks_shield_set (FALSE);
-}
-
-/**********************************************************************
- * Beeps (or not) at the user, in the tutor window
- */
-void
-tutor_beep ()
-{
-	GtkWidget *wg;
-
-	wg = get_wg ("togglebutton_tutor_beep");
-	if ((GTK_TOGGLE_BUTTON (wg))->active)
-		gdk_beep ();
-}
-
 /**********************************************************************
  * Load the list of files to include in the set of "other exercises"
  */
@@ -1469,3 +1506,210 @@ tutor_other_rename (const gchar *new_tx, const gchar *old_tx)
 		g_free (new_file);
 	}
 }
+
+/**********************************************************************
+ * Put 'mesg' in the message entry line of the shared tutor window
+ */
+void
+tutor_message (gchar * mesg)
+{
+	gint pos = 0;
+	GtkWidget *wg;
+
+	if (mesg == NULL)
+	{
+		g_message ("tutor_message() --> not showing NULL message!");
+		return;
+	}
+
+	wg = get_wg ("entry_mesg");
+	callbacks_shield_set (TRUE);
+	gtk_editable_delete_text (GTK_EDITABLE (wg), 0, -1);
+	gtk_editable_insert_text (GTK_EDITABLE (wg), g_strdup (mesg), strlen (mesg), &pos);
+	gtk_editable_set_position (GTK_EDITABLE (wg), -1);
+	callbacks_shield_set (FALSE);
+}
+
+/**********************************************************************
+ * Beeps (or not) at the user, in the tutor window
+ */
+void
+tutor_beep ()
+{
+	GtkWidget *wg;
+
+	wg = get_wg ("togglebutton_tutor_beep");
+	if ((GTK_TOGGLE_BUTTON (wg))->active)
+		gdk_beep ();
+}
+
+/**********************************************************************
+ * Speak some phrase
+ */ 
+void
+tutor_speak_string (gchar *string, gboolean wait)
+{
+	gchar *tmp_code;
+	gchar *command;
+	static gboolean espeak_OK = TRUE;
+	static GtkWidget *wg = NULL;
+
+	if (wg == NULL)
+		wg = get_wg ("checkbutton_speech");
+
+	if (espeak_OK == FALSE)
+		return;
+	if (!GTK_WIDGET_VISIBLE (wg) )
+		return;
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wg)) )
+		return;
+
+	/* Translators: your language code (first 2 letters of your po-file)*/
+	tmp_code = g_strdup (_("en"));
+	tmp_code[2] = '\0';
+	if (wait)
+	{
+		command = g_strdup_printf ("espeak -v%s -k1 \"%s\"", tmp_code, string);
+#ifdef G_OS_UNIX
+		espeak_OK = g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
+#else
+		espeak_OK = ! system (command);
+#endif
+	}
+	else
+	{
+#ifdef G_OS_UNIX
+		if (g_utf8_strlen (string, -1) == 1 || tutor.type == TT_VELO)
+			command = g_strdup_printf ("espeak -v%s -k1 --punct '%s'", tmp_code, string);
+		else
+			command = g_strdup_printf ("espeak -v%s -k1 \"%s\"", tmp_code, string);
+		espeak_OK = g_spawn_command_line_async (command, NULL);
+#else
+		if (g_utf8_strlen (string, -1) == 1 || tutor.type == TT_VELO)
+			command = g_strdup_printf ("espeak -v%s -k1 --punct \"%s\"", tmp_code, string);
+		else
+			command = g_strdup_printf ("espeak -v%s -k1 \"%s\"", tmp_code, string);
+		espeak_OK = ! system (command);
+#endif
+	}
+	if (espeak_OK == FALSE)
+		g_message ("Espeak not installed, so we'll say nothing:\n %s", command);
+	g_free (tmp_code);
+	g_free (command);
+}
+
+/**********************************************************************
+ * Control delayed tips for the finger to be used
+ */ 
+gboolean
+tutor_delayed_finger_tip (gpointer unich)
+{
+	gchar *finger;
+	gunichar *uch = (gunichar*) unich;
+	static gint counter = 0;
+
+	if (unich == NULL)
+	{
+		counter++;
+		return FALSE;
+	}
+	counter--;
+
+	if (counter > 0)
+		return FALSE;
+
+	finger = hints_finger_name_from_char (*uch);
+	tutor_speak_string (finger, TRUE);
+	g_free (finger);
+
+	return FALSE;
+}
+
+/**********************************************************************
+ * Speak the current character to be typed
+ */
+void
+tutor_speak_char ()
+{
+	gchar ut8[100];
+	static gunichar uch;
+
+	if (tutor.type == TT_BASIC)
+	{
+		g_timeout_add (3000, (GSourceFunc) tutor_delayed_finger_tip, (gpointer) &uch);
+		tutor_delayed_finger_tip (NULL);
+	}
+
+	uch = cursor_get_char ();
+	switch (uch)
+	{
+	case L' ':
+		strcpy (ut8, _("space"));
+		break;
+	case L'y':
+	case L'Y':
+		/* Translators: the name of letter Y */
+		strcpy (ut8, _("wye"));
+		break;
+	case UPSYM:
+		/* Translators: the name of the Return key */
+		strcpy (ut8, _("enter"));
+		break;
+	case L'%':
+		strcpy (ut8, "%");
+		break;
+	case L'\'':
+		strcpy (ut8, _("apostrophe"));
+		break;
+	case L'\"':
+		/* Translators: double quote symbol: " */
+		strcpy (ut8, _("quote"));
+		break;
+	case L'&':
+		/* Translators: ampersand symbol: & */
+		strcpy (ut8, _("ampersand"));
+		break;
+	default:
+		ut8[g_unichar_to_utf8 (uch, ut8)] = '\0';
+	}
+
+	tutor_speak_string (ut8, FALSE);
+
+}
+
+/**********************************************************************
+ * Speak the next word to be typed
+ */
+void
+tutor_speak_word ()
+{
+	gunichar uch[100];
+	gchar *ut8;
+	gint i;
+
+	if (cursor_advance (-1) != -1)
+		uch[0] = L' ';
+	else
+	{
+		uch[0] = cursor_get_char ();
+		cursor_advance (1);
+	}
+	if (uch[0] == L' ' || uch[0] == UPSYM || uch[0] == L'\n' || uch[0] == L'\r')
+	{
+		for (i = 0; i < 100; i++)
+		{
+			uch[i] = cursor_get_char ();
+			if (uch[i] == L' ' || uch[i] == UPSYM || uch[i] == L'\n' || uch[i] == L'\r')
+				break;
+			cursor_advance (1);
+		} 
+		cursor_advance (-i);
+	}
+	else
+		return;
+
+	ut8 = g_ucs4_to_utf8 (uch, i, NULL, NULL, NULL);
+	if (ut8)
+		tutor_speak_string (ut8, FALSE);
+	g_free (ut8);
+}
