@@ -10,11 +10,12 @@
 /*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
 /*****************************************************************************/
 
+#ifndef TUTOR_H
+# define TUTOR_H
+
 /*
  * Special colors
  */
-#ifndef TUTOR_H
-# define TUTOR_H
 # define TUTOR_RED "#ff0044"
 # define TUTOR_RED_LITE "#eeddbb"
 # define TUTOR_GREEN "#228822"
@@ -101,12 +102,20 @@ void tutor_char_distribution_count (gchar * text, Char_Distribution * dist);
 
 void tutor_draw_paragraph (gchar * text);
 
-void tutor_message (gchar * mesg);
-
-void tutor_beep (void);
-
 void tutor_load_list_other (gchar * file_name_end, GtkListStore * list);
 
 void tutor_other_rename (const gchar *new_tx, const gchar *old_tx);
 
+void tutor_message (gchar * mesg);
+
+void tutor_beep (void);
+
+gboolean tutor_delayed_finger_tip (gpointer unich);
+
+void tutor_speak_string (gchar *string, gboolean wait);
+
+void tutor_speak_char ();
+
+void tutor_speak_word ();
+
 #endif
